/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/clock_control.h>
#include <zephyr/drivers/clock_control/nrf_clock_control.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/irq.h>
#include <zephyr/logging/log.h>
#include <nrf.h>
#include <esb.h>
#include <zephyr/kernel.h>
#include <zephyr/types.h>

LOG_MODULE_REGISTER(esb_prx, CONFIG_ESB_PRX_APP_LOG_LEVEL);

// Configure payload
static struct esb_payload rx_payload;

void event_handler(struct esb_evt const *event)
{
	switch (event->evt_id) {
	case ESB_EVENT_TX_SUCCESS:
		LOG_DBG("TX SUCCESS EVENT");
		break;
	case ESB_EVENT_TX_FAILED:
		LOG_DBG("TX FAILED EVENT");
		break;
	case ESB_EVENT_RX_RECEIVED:
		if (esb_read_rx_payload(&rx_payload) == 0) {
			LOG_DBG("Packet received, len %d, pid %d, pipe %d, crc 0x%04x, rssi -%ddBm : "
				"0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, "
				"0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, "
				"0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, "
				"0x%02x, 0x%02x", 
				rx_payload.length, 
				rx_payload.pid,
				rx_payload.pipe,
				nrf_radio_rxcrc_get(NRF_RADIO),
				rx_payload.rssi,
				rx_payload.data[0],rx_payload.data[1],rx_payload.data[2],rx_payload.data[3],rx_payload.data[4],rx_payload.data[5],rx_payload.data[6],rx_payload.data[7],rx_payload.data[8],rx_payload.data[9],
				rx_payload.data[10],rx_payload.data[11],rx_payload.data[12],rx_payload.data[13],rx_payload.data[14],rx_payload.data[15],rx_payload.data[16],rx_payload.data[17],rx_payload.data[18],rx_payload.data[19],
				rx_payload.data[20],rx_payload.data[21],rx_payload.data[22],rx_payload.data[23],rx_payload.data[24],rx_payload.data[25],rx_payload.data[26],rx_payload.data[27],rx_payload.data[28],rx_payload.data[29],
				rx_payload.data[30],rx_payload.data[31]		
				);
		} else {
			LOG_ERR("Error while reading rx packet");
		}
		break;
	}
}

int clocks_start(void)
{
	int err;
	int res;
	struct onoff_manager *clk_mgr;
	struct onoff_client clk_cli;

	clk_mgr = z_nrf_clock_control_get_onoff(CLOCK_CONTROL_NRF_SUBSYS_HF);
	if (!clk_mgr) {
		LOG_ERR("Unable to get the Clock manager");
		return -ENXIO;
	}

	sys_notify_init_spinwait(&clk_cli.notify);

	err = onoff_request(clk_mgr, &clk_cli);
	if (err < 0) {
		LOG_ERR("Clock request failed: %d", err);
		return err;
	}

	do {
		err = sys_notify_fetch_result(&clk_cli.notify, &res);
		if (!err && res) {
			LOG_ERR("Clock could not be started: %d", res);
			return res;
		}
	} while (err);

	LOG_DBG("HF clock started");
	return 0;
}

int esb_initialize(void)
{
	int err;

	// All addresses will be Rx in this program
	uint8_t base_addr_0[3] = {0xE7, 0xE7, 0xE7};
	uint8_t base_addr_1[3] = {0xF0, 0xF0, 0xF0};
	// Prefix for pipes
	uint8_t addr_prefix[8] = {0xE7, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8};

	// Declare ESB configuration. This must match the Tx config
	struct esb_config config = ESB_LEGACY_CONFIG;

	//Configure for Rx
	config.mode = ESB_MODE_PRX;
	config.protocol = ESB_PROTOCOL_ESB_DPL;
	config.event_handler = event_handler;
	config.crc = ESB_CRC_16BIT;

	// Init ESB
	err = esb_init(&config);
	if (err) {
		return err;
	}

	// Set address length. Address consists of base address + 1-byte prefix
	err = esb_set_address_length(sizeof(base_addr_0)+1);
	if (err) {
		return err;
	}

	// Configure addresses
	err = esb_set_base_address_0(base_addr_0);
	if (err) {
		return err;
	}

	err = esb_set_base_address_1(base_addr_1);
	if (err) {
		return err;
	}

	err = esb_set_prefixes(addr_prefix, ARRAY_SIZE(addr_prefix));
	if (err) {
		return err;
	}

	return 0;
}

int main(void)
{
	int err;

	LOG_INF("Enhanced ShockBurst prx sample");

	err = clocks_start();
	if (err) {
		return 0;
	}

	err = esb_initialize();
	if (err) {
		LOG_ERR("ESB initialization failed, err %d", err);
		return 0;
	}

	LOG_INF("Initialization complete");
	LOG_INF("Setting up for packet receiption");

	err = esb_start_rx();
	if (err) {
		LOG_ERR("RX setup failed, err %d", err);
		return 0;
	}

	/* return to idle thread */
	return 0;
}
